package be.iminds.ilabt.jfed.rspec.model.imutable_impl;

import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.rspec.model.*;
import be.iminds.ilabt.jfed.rspec.parser.EventListExtraXml;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import be.iminds.ilabt.jfed.rspec.parser.RspecParseException;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.xml.stream.XMLEventWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

public class ImmutableRspecFactory implements RspecFactory {
    private final static Logger LOG = LoggerFactory.getLogger(ImmutableRspecFactory.class);

    private static final ImmutableRspecFactory INSTANCE = new ImmutableRspecFactory();

    public static ImmutableRspecFactory getInstance() {
        return INSTANCE;
    }

    private ImmutableRspecFactory() {
    }

    @Nonnull
    @Override
    public ModelRspecType getModelRspecType() {
        return ModelRspecType.IMMUTABLE;
    }

    @Nonnull
    @Override
    public ImmutableRspecNode createNode(@Nonnull ModelRspec modelRspec) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecNode createNodeWithClientId(@Nonnull ModelRspec modelRspec, @Nullable String id) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecNode copyNode(@Nonnull ModelRspec newModelRspec, @Nonnull RspecNode rspecNode, @Nonnull RspecNode.InterfaceCopyMethod interfaceCopyMethod) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecNode copyNodeIncludingInterfaces(@Nonnull ModelRspec newModelRspec, @Nonnull RspecNode rspecNode) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public GeantTestbedType copyGeantTestbedType(@Nonnull ModelRspec modelRSpec, @Nonnull GeantTestbedType gtt) {
        return new ImmutableGeantTestbedType(gtt);
    }

    @Nonnull
    @Override
    public RspecNode convertToSpecialNode(@Nonnull RspecNode origNode) throws RspecParseException {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecLink createLink(@Nonnull ModelRspec modelRspec, @Nullable String addDefaultLinkType) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecLink createLinkWithClientId(@Nonnull ModelRspec modelRspec, @Nonnull String id, @Nullable String addDefaultLinkType) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecLink copyLink(@Nonnull ModelRspec newModelRspec, @Nonnull RspecLink rspecLink, @Nonnull RspecNode.InterfaceCopyMethod interfaceCopyMethod) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecInterface createInterface(@Nonnull RspecNode node, @Nullable String id) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecInterface createInterface(@Nonnull RspecLink link, @Nullable String id) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecInterface createInterface(@Nonnull RspecNode node, @Nonnull RspecLink link, @Nullable String id) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableRspecInterface createInterface(@Nonnull RspecNode node, @Nonnull RspecLink link) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public LinkSetting createLinkSetting(@Nonnull RspecInterface fromIface, @Nonnull RspecInterface toIface) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ExecuteService createExecuteService(@Nullable String shell, @Nullable String command) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public InstallService createInstallService(@Nullable String installPath, @Nullable String url) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public AnsibleService createAnsibleService() {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableModelRspec createModelRspec(@Nullable String rspecType) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ImmutableModelRspec copyModelRspec(@Nonnull ModelRspec orig, @Nullable String newType) {
        assert newType == null || newType.equals(orig.getType());
        return new ImmutableModelRspec(this, orig);
    }

    @Override
    public boolean processExtraRspecXml(@Nonnull ModelRspec model, @Nonnull EventListExtraXml extraXml) throws RspecParseException {
        return false;
    }

    @Override
    public boolean processExtraNodeXml(@Nonnull RspecNode node, @Nonnull EventListExtraXml extraXml) throws RspecParseException {
        return false;
    }

    @Override
    public boolean processExtraLinkXml(@Nonnull RspecLink link, @Nonnull EventListExtraXml extraXml) throws RspecParseException {
        return false;
    }

    @Override
    public void writeExtraRspecXml(@Nonnull ModelRspec modelRspec, @Nonnull XMLEventWriter eventWriter) {
    }

    @Override
    public void writeExtraNodeXml(@Nonnull RspecNode node, @Nonnull XMLEventWriter eventWriter) {
    }

    @Override
    public void writeExtraLinkXml(@Nonnull RspecLink link, @Nonnull XMLEventWriter eventWriter) {
    }


    @Override
    public boolean equals(Object o) {
        return o instanceof ImmutableRspecFactory;
    }

    @Override
    public int hashCode() {
        return ImmutableRspecFactory.class.hashCode();
    }

    @Override
    public void mergeExtra(@Nonnull ModelRspec mergedModel, @Nonnull ModelRspec partialRspec) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void finishParsing(@Nonnull ModelRspec modelRspec) {

    }

    @Nonnull
    @Override
    public OpenflowDataPath createOpenflowDataPath(@Nonnull ModelRspec modelRspec, @Nonnull EventListExtraXml allXml, @Nullable String componentManagerId, @Nullable String componentId, @Nullable String dpidAttValue) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public Lease createLease(@Nonnull ModelRspec rspec, @Nullable String leaseId, @Nullable String clientId, @Nullable Date validFromDate, @Nullable Date validUntilDate, @Nullable String validFromString, @Nullable String validUntilString) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public Lease createLease(@Nonnull ModelRspec rspec, @Nullable String leaseId, @Nullable String clientId, @Nullable Date validFromDate, @Nullable Date validUntilDate) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public Channel createChannel(@Nonnull ModelRspec modelRspec) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public GeantTestbedType createGeantTestbedType(@Nullable String name) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public AddressPool createAddressPool(@Nonnull ModelRspec modelRspec) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public AddressPool copyAddressPool(@Nonnull ModelRspec modelRspec, @Nonnull AddressPool addressPool) {
        assert modelRspec instanceof ImmutableAddressPool;
        return new ImmutableAddressPool(addressPool);
    }

    @Nonnull
    @Override
    public IPv4 createIPv4(@Nonnull ModelRspec modelRspec) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public IPv4 createIPv4(@Nonnull ModelRspec modelRspec, @Nullable String address, @Nullable String netmask) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public ExecuteAnsiblePlaybook createExecuteAnsibleCookbook(@Nonnull ModelRspec rspec, @Nullable String source, @Nullable String outputFilename) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public DistributeSshKeypair createDistributeSshKeypair(@Nonnull ModelRspec rspec, @Nullable String location, @Nullable GeniUrn userUrn) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public GeantTestbedType.VpnService createGtsVpnService(String endpoint, String username, String password) {
        throw new UnsupportedOperationException();
    }

    @Nonnull
    @Override
    public WebVncService createWebVncService(String providerId, String vncUrl, String username, String password) {
        throw new UnsupportedOperationException();
    }
}
